# noinspection PyCompatibility
from queue import Queue, Empty
import datetime


class MyQueue(Queue):

    running = True
    defaultColor = '#7F7F7F'
    ccn = {
        u'0':    defaultColor,
        u'0;30': '#000000',
        u'0;31': '#FF0000',
        u'0;32': '#00FF00',
        u'0;33': '#FFFF00',
        u'0;34': '#0000FF',
        u'0;35': '#FF00FF',
        u'0;36': '#00FFFF',
        u'0;37': '#808080',
        u'1;30': '#404040',
        u'1;31': '#FF4500',
        u'1;32': '#ADFF2F',
        u'1;33': '#FFF68F',
        u'1;34': '#1E90FF',
        u'1;35': '#FF83FA',
        u'1;36': '#97FFFF',
        u'1;37': '#FFFFFF',
    }
    prefix = '['
    suffix = 'm'
    sysMsgColorCode = prefix + u'1;36' + suffix
    timeFormat = "%Y-%m-%d %H:%M:%S"
    tabs = []


    def getColor(self, code):
        if code in self.ccn:
            return self.ccn[code]
        return self.defaultColor


    def finishOutput(self):
        self.running = False


    def initCompany(self, name):
        pass


    def setCompanyName(self, company_id, name):
        pass


    def completeCompany(self, company_id, name):
        pass


    def newCompany(self, name):
        company_id, t = self.initCompany(name)
        tab = {
            'id': company_id,
            'name': name,
            'text': t,
            'tags': {},
        }
        self.tabs.append(tab)
        return tab


    def getCompany(self, name):
        for tab in self.tabs:
            if tab['name'] and tab['name'] == name:
                return tab
            if not tab['name']:
                tab['name'] = name
                self.setCompanyName(tab['id'], name)
                return tab
        return self.newCompany(name)


    def createTag(self, stream, tagname, **kwargs):
        pass


    def outputText(self, stream, text, tags, tagname, isnew):
        pass


    def processIncoming(self):
        while self.qsize():
            try:
                item = self.get(0)
                tab = self.getCompany(item['company'])
                time = datetime.datetime.now()
                if 'started' in item:
                    msg = '\n%s[%s] ###Started###' % (self.sysMsgColorCode, time.strftime(self.timeFormat))
                    tab['started'] = time
                elif 'completed' in item:
                    duration = time - tab['started']
                    msg = '%s[%s] [Duration: %s] ###Completed###\n' % (self.sysMsgColorCode, time.strftime(self.timeFormat), str(duration))
                    self.completeCompany(tab['id'], tab['name'])
                else:
                    msg = item['msg']
                color = self.defaultColor

                while True:
                    output = msg
                    next_pos = msg.find(self.prefix)
                    if next_pos > 0:
                        output = msg[:next_pos]
                        msg = msg[next_pos:]
                    else:
                        for c in self.ccn:
                            ansi = "%s%s%s" % (self.prefix, c, self.suffix)
                            if msg.find(ansi) == 0:
                                output = ''
                                msg = msg[len(ansi):]
                                color = self.getColor(c)

                    if output:
                        leading_spaces = len(output) - len(output.lstrip(' '))
                        prefix = '\n' + output[0:leading_spaces] + '    '
                        lines = output.split('\\n')
                        text = prefix.join(lines)

                        tags = tab['tags']
                        tagname = color.replace(' ', '_')
                        isnew = False
                        if tagname not in tags:
                            isnew = True
                            tags[tagname] = self.createTag(tab['text'], tagname, foreground=color)
                        self.outputText(tab['text'], text, tags, tagname, isnew)
                        if output == msg:
                            break
            except Empty:
                pass
