import httplib # Used only for handling httplib.HTTPException (case #26701)
import logging
import logging.handlers
import re
import urllib
import urllib2
import csv

# Build the request headers
headers = {
        'User-Agent': 'Server Density Agent',
        'Content-Type': 'application/x-www-form-urlencoded',
        'Accept': 'text/html, */*',
}

class HAProxy:
    def __init__( self, agentConfig, mainLogger, rawConfig ):
        self.agentConfig = agentConfig
        self.mainLogger = mainLogger
        self.rawConfig = rawConfig

    def myrun( self ):

        if 'haproxy_url' not in self.rawConfig['Main']:
            self.mainLogger.error("haproxy_url MUST be in the config")
            return False
        if 'haproxy_svname' not in self.rawConfig['Main']:
            self.mainLogger.error("haproxy_svname MUST be in the config")
            return False

        svname = self.rawConfig['Main']['haproxy_svname']

        if 'haproxy_username' in self.rawConfig['Main'] and self.rawConfig['Main']['haproxy_username'] != '':
            password_mgr = urllib2.HTTPPasswordMgrWithDefaultRealm()
            password_mgr.add_password(None, self.rawConfig['Main']['haproxy_url'], self.rawConfig['Main']['haproxy_username'], self.rawConfig['Main']['haproxy_password'])
            handler = urllib2.HTTPBasicAuthHandler(password_mgr)
            opener = urllib2.build_opener(handler)
            urllib2.install_opener(opener)
            pass

        req = urllib2.Request(self.rawConfig['Main']['haproxy_url'] + ';csv;norefresh', None, headers)
        request = urllib2.urlopen(req)
        response = request.read()

        # Parse out the returned CSV data
        reader = csv.DictReader(p.stdout)
        output = []
        for row in reader:
            output.append(row)
            pass

        # Now find the row that has our svname
        scur = 0
        wredis = 0
        stot = 0
        for i in output:
            if i['svname'] == svname:
                # Pull out scur, wredis, and stot
                scur = i['scur']
                wredis = i['wredis']
                stot = i['stot']

        return {'scur': scur, 'wredis': wredis, 'stot': stot}

    def run( self ):
        try:
            return self.myrun()
        except urllib2.HTTPError as e:
            self.mainLogger.error('Unable to get HAProxy status - HTTPError = ' + str(e))
            return False
        except urllib2.URLError as e:
            self.mainLogger.error('Unable to get HAProxy status - URLError = ' + str(e))
            return False
        
        except httplib.HTTPException as e:
            self.mainLogger.error('Unable to get HAProxy status - HTTPException = ' + str(e))
            return False
        
        except Exception as e:
            import traceback
            self.mainLogger.error('Unable to get HAProxy status - Exception = ' + traceback.format_exc())
            return False
